"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("./base");
/* eslint-disable */
var iso = new RegExp(/^(?:[-+]\d{2})?(?:\d{4}(?!\d{2}\b))(?:(-?)(?:(?:0[1-9]|1[0-2])(?:\1(?:[12]\d|0[1-9]|3[01]))?|W(?:[0-4]\d|5[0-2])(?:-?[1-7])?|(?:00[1-9]|0[1-9]\d|[12]\d{2}|3(?:[0-5]\d|6[1-6])))(?![T]$|[T][\d]+Z$)(?:[T\s](?:(?:(?:[01]\d|2[0-3])(?:(:?)[0-5]\d)?|24\:?00)(?:[.,]\d+(?!:))?)(?:\2[0-5]\d(?:[.,]\d+)?)?(?:[Z]|(?:[+-])(?:[01]\d|2[0-3])(?::?[0-5]\d)?)?)?)?$/);
/* eslint-enable */
exports.messages = {
    'date.typeof': function (name) { return name + " must be a valid date"; },
    'date.iso': function (name) { return name + " must be an ISO-8601 date"; },
    'date.min': function (name, _a) {
        var min = _a.min;
        return name + " must be at least " + min.toISOString();
    },
    'date.max': function (name, _a) {
        var max = _a.max;
        return name + " must be less than or equal to " + max.toISOString();
    },
};
/**
 * Date class
 */
var TypeDate = /** @class */ (function (_super) {
    __extends(TypeDate, _super);
    /**
     * Constructor
     */
    function TypeDate() {
        var _this = _super.call(this) || this;
        _this.name = 'date';
        _this.typeof();
        return _this;
    }
    /**
     * Try to cast value
     *
     * @param {mixed} value
     * @return {Date|mixed}
     */
    TypeDate.prototype._cast = function (value) {
        return Date.parse(value);
    };
    /**
     * Test to validate the type of the value
     *
     * @return {this}
     */
    TypeDate.prototype.typeof = function () {
        this.test('typeof', function (str) {
            return !isNaN(Date.parse(str));
        });
        return this;
    };
    /**
     * Force a date to be a valid ISO-8601.
     *
     * @return {this}
     */
    TypeDate.prototype.iso = function () {
        this.test('iso', function (str) {
            return str.match(iso) !== null;
        });
        return this;
    };
    /**
     * Force a date to be a at least or bigger than value passed
     *
     * @param  {Date} min
     * @return {this}
     */
    TypeDate.prototype.min = function (min) {
        this.test('min', function (str) {
            if (typeof min !== 'object' ||
                min.constructor.name !== 'Date' ||
                min.toString() === 'Invalid Date') {
                return false;
            }
            return Date.parse(str) >= min.getTime();
        }, { min: min });
        return this;
    };
    /**
     * Force a date to be less or equal than value passed
     *
     * @param  {Date} max
     * @return {this}
     */
    TypeDate.prototype.max = function (max) {
        this.test('max', function (str) {
            if (typeof max !== 'object' ||
                max.constructor.name !== 'Date' ||
                max.toString() === 'Invalid Date') {
                return false;
            }
            return Date.parse(str) <= max.getTime();
        }, { max: max });
        return this;
    };
    return TypeDate;
}(base_1.default));
exports.TypeDate = TypeDate;
var def = {
    Class: TypeDate,
    messages: exports.messages,
};
exports.default = def;
