"use strict";
var isPlainObject_1 = require("lodash/isPlainObject");
var lang_1 = require("./lang");
var validator_1 = require("./validator");
var base_1 = require("./base");
// Default extractors
var any_1 = require("./any");
var array_1 = require("./array");
var boolean_1 = require("./boolean");
var date_1 = require("./date");
var function_1 = require("./function");
var internet_1 = require("./internet");
var number_1 = require("./number");
var object_1 = require("./object");
var string_1 = require("./string");
var Instance = function (lang) {
    // @ts-ignore
    var inst = function (schema) {
        return new validator_1.default(schema, inst);
    };
    inst.langList = Object.assign({}, lang_1.default);
    inst.templates = {};
    inst.Base = base_1.default;
    inst.instance = function (newLang) {
        if (newLang === void 0) { newLang = {}; }
        return Instance(Object.assign({}, inst.langList, newLang));
    };
    inst.use = function (plugin) {
        var test = new plugin.Class();
        inst[test.name || test.constructor.name] = function () {
            return new plugin.Class(inst);
        };
        if (plugin.messages) {
            inst.lang(plugin.messages);
        }
        return inst;
    };
    inst.lang = function (key, tpl) {
        if (isPlainObject_1.default(key)) {
            inst.langList = Object.assign({}, inst.langList, key);
        }
        else if (typeof key === 'string' && tpl) {
            inst.langList[key] = tpl;
        }
    };
    inst.template = function (name, schema) {
        inst.templates[name] = schema;
    };
    inst.is = function (name) {
        if (typeof inst.templates[name] === 'undefined') {
            throw new Error("unknow template " + name);
        }
        return inst.templates[name].clone();
    };
    inst.formatError = function (_a, label, position) {
        var type = _a.type, args = _a.args, result = _a.result;
        if (label === void 0) { label = 'value'; }
        // Get messages from error
        var msg;
        if (typeof inst.langList[type] !== 'undefined') {
            msg = inst.langList[type](label, args, result);
        }
        else {
            msg = 'Invalid value';
        }
        var formatted = {
            label: label,
            type: type,
            message: msg,
        };
        if (typeof position === 'number') {
            formatted.position = position;
        }
        // nested errors
        if (result && result.errors) {
            formatted.errors = result.errors.map(function (error) {
                return inst.formatError(error.test, error.label, error.position);
            });
        }
        return formatted;
    };
    // Declare basic plugins
    inst.use(any_1.default);
    inst.use(array_1.default);
    inst.use(boolean_1.default);
    inst.use(date_1.default);
    inst.use(function_1.default);
    inst.use(internet_1.default);
    inst.use(number_1.default);
    inst.use(object_1.default);
    inst.use(string_1.default);
    // Add passed lang object
    if (lang && isPlainObject_1.default(lang)) {
        inst.lang(lang);
    }
    return inst;
};
var def = Instance();
module.exports = def;
