"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("./base");
var luhn_1 = require("./utils/luhn");
var url_1 = require("url");
// eslint-disable-next-line
var hostname = new RegExp(/^(?=.{1,255}$)[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?(?:\.[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?)*\.?$/);
var uuidv4 = new RegExp(/^[0-9a-f]{8}-?[0-9a-f]{4}-?[1-5][0-9a-f]{3}-?[89ab][0-9a-f]{3}-?[0-9a-f]{12}$/i);
var ipv4 = new RegExp(/^(?!0)(?!.*\.$)((1?\d?\d|25[0-5]|2[0-4]\d)(\.|$)){4}$/);
var ipv6 = new RegExp(/^((?:[0-9A-Fa-f]{1,4}))((?::[0-9A-Fa-f]{1,4}))*::((?:[0-9A-Fa-f]{1,4}))((?::[0-9A-Fa-f]{1,4}))*|((?:[0-9A-Fa-f]{1,4}))((?::[0-9A-Fa-f]{1,4})){7}$/);
exports.messages = {
    'internet.typeof': function (name) { return name + " must be a valid string"; },
    'internet.url': function (name) { return name + " must be a valid url"; },
    'internet.hostname': function (name) { return name + " must be a valid hostname"; },
    'internet.hex': function (name) { return name + " must be a valid hex"; },
    'internet.creditCard': function (name) { return name + " must be a valid Credit Card"; },
    'internet.uuidv4': function (name) { return name + " must be a valid token"; },
    'internet.ip': function (name) { return name + " must be a valid IP"; },
    'internet.ipv4': function (name) { return name + " must be a valid IP v4"; },
    'internet.ipv6': function (name) { return name + " must be a valid IP v6"; },
};
/**
 * Internet class
 */
var TypeInternet = /** @class */ (function (_super) {
    __extends(TypeInternet, _super);
    /**
     * Constructor
     */
    function TypeInternet() {
        var _this = _super.call(this) || this;
        _this.name = 'internet';
        _this.typeof();
        return _this;
    }
    /**
     * Test to validate the type of the value
     *
     * @return {this}
     */
    TypeInternet.prototype.typeof = function () {
        this.test('typeof', function (str) {
            return typeof str === 'string';
        });
        return this;
    };
    /**
     * Force a string to be a valid url (RFC)
     *
     * @return {this}
     */
    TypeInternet.prototype.url = function () {
        this.test('url', function (str) {
            try {
                // eslint-disable-next-line
                if (str.search('javascript:') === 0) {
                    return false;
                }
                // eslint-disable-next-line
                new url_1.default.URL(str);
                return true;
            }
            catch (e) {
                return false;
            }
        });
        return this;
    };
    /**
     * Force a string to be a valid hostname (RFC)
     *
     * @return {this}
     */
    TypeInternet.prototype.hostname = function () {
        this.test('hostname', function (str) {
            return str.match(hostname) !== null;
        });
        return this;
    };
    /**
     * Force a string to be a valid hex (a-f0-9)
     *
     * @return {this}
     */
    TypeInternet.prototype.hex = function () {
        this.test('hex', function (str) {
            return str.match(/^[a-f0-9]+$/i) !== null;
        });
        return this;
    };
    /**
     * Force a string to be a valid credit card (using Luhn's algorithm)
     *
     * @return {this}
     */
    TypeInternet.prototype.creditCard = function () {
        this.test('creditCard', function (str) {
            if (/[^0-9-\s]+/.test(str)) {
                return false;
            }
            str = str.replace(/\D/g, '');
            if (str.length < 13) {
                return false;
            }
            return luhn_1.default(str) === true;
        });
        return this;
    };
    /**
     * Force a string to be a valid uuid version 4
     *
     * @return {this}
     */
    TypeInternet.prototype.uuidv4 = function () {
        this.test('uuidv4', function (str) {
            if (str.length < 32) {
                return false;
            }
            return uuidv4.test(str);
        });
        return this;
    };
    /**
     * Force a string to be a valid ipv4 or ipv6
     *
     * @return {this}
     */
    TypeInternet.prototype.ip = function () {
        this.test('ip', function (str) {
            return ipv4.test(str) || ipv6.test(str);
        });
        return this;
    };
    /**
     * Force a string to be a valid ipv4
     *
     * @return {this}
     */
    TypeInternet.prototype.ipv4 = function () {
        this.test('ipv4', function (str) {
            return ipv4.test(str);
        });
        return this;
    };
    /**
     * Force a string to be a valid ipv6
     *
     * @return {this}
     */
    TypeInternet.prototype.ipv6 = function () {
        this.test('ipv6', function (str) {
            return ipv6.test(str);
        });
        return this;
    };
    return TypeInternet;
}(base_1.default));
exports.TypeInternet = TypeInternet;
var def = {
    Class: TypeInternet,
    messages: exports.messages,
};
exports.default = def;
