"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("./base");
exports.messages = {
    'number.typeof': function (name) { return name + " must be a valid number"; },
    'number.min': function (name, args) { return name + " must be at least " + args.min; },
    'number.max': function (name, args) {
        return name + " must be less than or equal to " + args.max;
    },
    'number.integer': function (name) { return name + " must be an integer"; },
    'number.unsigned': function (name) { return name + " must be an unsigned number"; },
    'number.positive': function (name) { return name + " must be a positive number"; },
    'number.negative': function (name) { return name + " must be a negative number"; },
    'number.in': function (name, args) { return name + " must be one of [" + args.obj + "]"; },
    'number.not': function (name) { return name + " contains forbidden value"; },
};
/**
 * Number class
 */
var TypeNumber = /** @class */ (function (_super) {
    __extends(TypeNumber, _super);
    /**
     * Constructor
     */
    function TypeNumber() {
        var _this = _super.call(this) || this;
        _this.name = 'number';
        _this.typeof();
        return _this;
    }
    /**
     * Try to cast value
     *
     * @param {mixed} value
     * @return {Number|mixed}
     */
    TypeNumber.prototype._cast = function (value) {
        if (typeof value === 'string') {
            return parseFloat(value);
        }
        return value;
    };
    /**
     * Test to validate the type of the value
     *
     * @return {this}
     */
    TypeNumber.prototype.typeof = function () {
        this.test('typeof', function (str) {
            return typeof str === 'number' && !isNaN(str) && isFinite(str);
        });
        return this;
    };
    /**
     * Force a number to be equal or more to the value passed.
     *
     * @param  {number} min
     * @return {this}
     */
    TypeNumber.prototype.min = function (min) {
        this.test('min', function (str) {
            return str >= min;
        }, { min: min });
        return this;
    };
    /**
     * Force a number to be equal or less to the value passed.
     *
     * @param  {number} max
     * @return {this}
     */
    TypeNumber.prototype.max = function (max) {
        this.test('max', function (str) {
            return str <= max;
        }, { max: max });
        return this;
    };
    /**
     * Force a number to be an integer.
     *
     * @return {this}
     */
    TypeNumber.prototype.integer = function () {
        this.test('integer', function (str) {
            return Number.isSafeInteger(str) === true;
        });
        return this;
    };
    /**
     * Force a number to be unsigned.
     *
     * @return {this}
     */
    TypeNumber.prototype.unsigned = function () {
        this.test('unsigned', function (str) {
            return str >= 0;
        });
        return this;
    };
    /**
     * Force a number to be greater than 0.
     *
     * @return {this}
     */
    TypeNumber.prototype.positive = function () {
        this.test('positive', function (str) {
            return str > 0;
        });
        return this;
    };
    /**
     * Force a number to be lesser than 0.
     *
     * @return {this}
     */
    TypeNumber.prototype.negative = function () {
        this.test('negative', function (str) {
            return str < 0;
        });
        return this;
    };
    /**
     * Force a number to be equal to one of the value passed in the set.
     *
     * @param  {...number} obj
     * @return {this}
     */
    TypeNumber.prototype.in = function () {
        var obj = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            obj[_i] = arguments[_i];
        }
        this.test('in', function (str) {
            return obj.includes(str) === true;
        }, { obj: obj });
        return this;
    };
    /**
     * Force a number to be different to all of the value passed in the set.
     *
     * @param  {...number} obj
     * @return {this}
     */
    TypeNumber.prototype.not = function () {
        var obj = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            obj[_i] = arguments[_i];
        }
        this.test('not', function (str) {
            return obj.includes(str) === false;
        }, { obj: obj });
        return this;
    };
    return TypeNumber;
}(base_1.default));
exports.TypeNumber = TypeNumber;
var def = {
    Class: TypeNumber,
    messages: exports.messages,
};
exports.default = def;
