"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("./base");
exports.messages = {
    'string.typeof': function (name) { return name + " must be a valid string"; },
    'string.empty': function (name) { return name + " can not be empty"; },
    'string.min': function (name, args) {
        return name + " must be at least " + args.min + " characters long";
    },
    'string.max': function (name, args) {
        return name + " must be at most " + args.max + " characters long";
    },
    'string.pattern': function (name, args) {
        return name + " must match pattern \"" + args.regex + "\"";
    },
    'string.in': function (name, args) { return name + " must be one of [" + args.obj + "]"; },
    'string.not': function (name) { return name + " contains forbidden value"; },
    'string.email': function (name) { return name + " must be a valid email"; },
    'string.lowercase': function (name) { return name + " must be lowercase only"; },
    'string.uppercase': function (name) { return name + " must be uppercase only"; },
};
/**
 * String class
 */
var TypeString = /** @class */ (function (_super) {
    __extends(TypeString, _super);
    /**
     * Constructor
     */
    function TypeString() {
        var _this = _super.call(this) || this;
        _this.name = 'string';
        _this._noEmpty = false;
        _this.typeof();
        return _this;
    }
    /**
     * Test to validate the type of the value
     *
     * @return {this}
     */
    TypeString.prototype.typeof = function () {
        this.test('typeof', function (str) {
            return typeof str === 'string';
        });
        return this;
    };
    /**
     * Force a string to be not empty
     *
     * @return {this}
     */
    TypeString.prototype.noEmpty = function () {
        this._noEmpty = true;
        this.test('empty', function (str) {
            return str.length > 0;
        });
        return this;
    };
    /**
     * Force a string to be equal or more to the value passed.
     *
     * @param  {number} min
     * @return {this}
     */
    TypeString.prototype.min = function (min) {
        this.test('min', function (str) {
            return str.length >= min;
        }, { min: min });
        return this;
    };
    /**
     * Force a string to be equal or less to the value passed.
     *
     * @param  {number} max
     * @return {this}
     */
    TypeString.prototype.max = function (max) {
        this.test('max', function (str) {
            return str.length <= max;
        }, { max: max });
        return this;
    };
    /**
     * Force a string to match the regex passed.
     *
     * @param  {Regex} regex
     * @return {this}
     */
    TypeString.prototype.pattern = function (regex) {
        this.test('pattern', function (str) {
            return str.match(regex) !== null;
        }, { regex: regex });
        return this;
    };
    /**
     * Force a string to be equal to one of the value passed in the set.
     *
     * @param  {...string} obj
     * @return {this}
     */
    TypeString.prototype.in = function () {
        var obj = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            obj[_i] = arguments[_i];
        }
        this.test('in', function (str) {
            return obj.includes(str) === true;
        }, { obj: obj });
        return this;
    };
    /**
     * Force a string to be different to all of the value passed in the set.
     *
     * @param  {...string} obj
     * @return {this}
     */
    TypeString.prototype.not = function () {
        var obj = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            obj[_i] = arguments[_i];
        }
        this.test('not', function (str) {
            return obj.includes(str) === false;
        }, { obj: obj });
        return this;
    };
    /**
     * Force a string to be a valid email (contain an @).
     *
     * @return {this}
     */
    TypeString.prototype.email = function () {
        this.test('email', function (str) {
            return str.search('@') >= 0;
        });
        return this;
    };
    /**
     * Force a string to be fully in lowercase.
     *
     * @return {this}
     */
    TypeString.prototype.lowercase = function () {
        this.test('lowercase', function (str) {
            return str.toLocaleLowerCase() === str;
        });
        return this;
    };
    /**
     * Force a string to be fully in uppercase.
     *
     * @return {this}
     */
    TypeString.prototype.uppercase = function () {
        this.test('uppercase', function (str) {
            return str.toLocaleUpperCase() === str;
        });
        return this;
    };
    return TypeString;
}(base_1.default));
exports.TypeString = TypeString;
var def = {
    Class: TypeString,
    messages: exports.messages,
};
exports.default = def;
