"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var isPlainObject_1 = require("lodash/isPlainObject");
var isEqual_1 = require("lodash/isEqual");
var object_1 = require("./object");
var createTest_1 = require("./utils/createTest");
/**
 * Validator class
 * new Validator({ foo: 'bar' });
 */
var Validator = /** @class */ (function () {
    /**
     * Constructor
     * @param  {object} schema
     * @param  {object} inst   An Altheia instance
     */
    function Validator(schema, inst) {
        this.inst = inst;
        this.isValidator = 1;
        this.validated = false;
        this._schema = {};
        this._body = {};
        this._errors = [];
        this._errorsRaw = [];
        this._confirm = [];
        this._options = {
            required: false,
            unknown: false,
            flatten: false,
        };
        this.schema(schema);
    }
    /**
     * Clone a validator
     * @return {Validator}
     */
    Validator.prototype.clone = function () {
        var clone = new Validator(this._schema, this.inst);
        clone.options(this._options);
        clone._confirm = this._confirm.slice();
        return clone;
    };
    /**
     * Assign the body to validate
     *
     * @param  {object} body
     * @return {this}
     */
    Validator.prototype.body = function (body) {
        this._body = Object.assign({}, body);
        return this;
    };
    /**
     * Declare the schema that describe the body()
     *
     * @param  {object} schema
     * @return {this}
     */
    Validator.prototype.schema = function (schema) {
        if (!isPlainObject_1.default(schema)) {
            throw new Error('schema should be object');
        }
        this._schema = schema;
        return this;
    };
    /**
     * Declare options to change the defaults behaviour of the Validator
     *
     * @param  {object} options
     * @return {this}
     */
    Validator.prototype.options = function (options) {
        if (!isPlainObject_1.default(options)) {
            throw new Error('schema should be object');
        }
        this._options = Object.assign(this._options, options);
        return this;
    };
    /**
     * Format any Error Array returned by a test
     * @param  {object} error
     * @param  {string} label
     * @return {ValidatorErrorFormatted} Formatted error
     */
    Validator.prototype.formatError = function (error, label) {
        return this.inst.formatError(error, label);
    };
    Validator.prototype.flatten = function (bag, error) {
        if (error.errors) {
            return error.errors.reduce(this.flatten, bag);
        }
        bag.push(error);
        return bag;
    };
    Validator.prototype.validate = function () {
        var params = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            params[_i] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var callback, returnOrCallback, errors, only, keys, i, key, item, value, hasError, formatted;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.validated) {
                            throw new Error('Already validated, please use .clone() to validate a different body');
                        }
                        callback = undefined;
                        if (params.length > 0) {
                            if (typeof params[0] === 'function') {
                                callback = params[0];
                            }
                            else {
                                this.body(params[0]);
                                callback = params[1];
                            }
                        }
                        this.validated = true;
                        returnOrCallback = function (result, callback) {
                            if (callback) {
                                callback(result);
                            }
                            return result;
                        };
                        errors = [];
                        if (!(this._options.unknown === false)) return [3 /*break*/, 2];
                        return [4 /*yield*/, new object_1.TypeObject()
                                .in(Object.keys(this._schema), { oneErrorPerKey: true })
                                .validate(this._body)];
                    case 1:
                        only = _a.sent();
                        if (typeof only !== 'boolean' && only.result && only.result.errors) {
                            only.result.errors.map(function (error) {
                                return errors.push(_this.formatError(error.test, error.label));
                            });
                        }
                        _a.label = 2;
                    case 2:
                        keys = Object.keys(this._schema);
                        i = 0;
                        _a.label = 3;
                    case 3:
                        if (!(i < keys.length)) return [3 /*break*/, 6];
                        key = keys[i];
                        item = this._schema[key];
                        value = typeof this._body[key] !== 'undefined' ? this._body[key] : null;
                        this._body[key] = value;
                        // If not required pass
                        if (this._options.required === true) {
                            item.required();
                        }
                        return [4 /*yield*/, item.validate(value)];
                    case 4:
                        hasError = _a.sent();
                        if (!hasError) {
                            return [3 /*break*/, 5];
                        }
                        this._errorsRaw.push({ test: hasError, label: key });
                        formatted = this.formatError(hasError, key);
                        if (!formatted.errors || !this._options.flatten) {
                            errors.push(formatted);
                        }
                        else if (this._options.flatten && formatted.errors) {
                            errors.push.apply(errors, formatted.errors.reduce(this.flatten, []));
                        }
                        _a.label = 5;
                    case 5:
                        i++;
                        return [3 /*break*/, 3];
                    case 6:
                        this._errors = errors;
                        // Check confirm after validation
                        if (this._confirm.length > 0) {
                            this._confirm.forEach(function (item) {
                                var initial = _this._body[item.initial];
                                var comparison = _this._body[item.comparison];
                                if (isEqual_1.default(initial, comparison)) {
                                    return;
                                }
                                _this._errors.push(_this.formatError(createTest_1.createTestResult(createTest_1.createTest({ type: 'confirm', args: item }), false), item.comparison));
                            });
                        }
                        if (errors.length > 0) {
                            return [2 /*return*/, returnOrCallback(errors, callback)];
                        }
                        return [2 /*return*/, returnOrCallback(false, callback)];
                }
            });
        });
    };
    /**
     * Force the confirmation of one field by an other one
     *
     * @param  {string} initial    The first key
     * @param  {string} comparison The second key
     * @return {this}
     */
    Validator.prototype.confirm = function (initial, comparison) {
        this._confirm.push({ initial: initial, comparison: comparison });
        return this;
    };
    return Validator;
}());
exports.default = Validator;
